@@ -23,10 +23,7 @@
 #ifndef _GTK_HTML_EDIT_IMAGE_H_
 #define _GTK_HTML_EDIT_IMAGE_H_
 
-typedef struct _GtkHTMLImageDialog GtkHTMLImageDialog;
-
 #include <gnome.h>
-#include <htmlimage.h>
 #include "control-data.h"
 
 GtkWidget          * image_properties              (GtkHTMLControlData *cd, gpointer *set_data);
