@@ -56,9 +56,10 @@ static struct {
 	{ GTK_HTML_PARAGRAPH_STYLE_H5, N_("Header 5") },
 	{ GTK_HTML_PARAGRAPH_STYLE_H6, N_("Header 6") },
 	{ GTK_HTML_PARAGRAPH_STYLE_ADDRESS, N_("Address") },
-	{ GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT, N_("List item (digit)") },
-	{ GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED, N_("List item (unnumbered)") },
-	{ GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN, N_("List item (roman)") },
+	{ GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED, N_("Bulleted List") },
+	{ GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT, N_("Numbered List") },
+	{ GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN, N_("Roman List") },
+	{ GTK_HTML_PARAGRAPH_STYLE_ITEMALPHA, N_("Alphabetical List") },
 	{ GTK_HTML_PARAGRAPH_STYLE_NORMAL, NULL },
 };
 
@@ -111,6 +112,7 @@ paragraph_style_menu_item_update (GtkWidget *widget, gpointer format_html)
 		     || style == GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED
 		     || style == GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN
 		     || style == GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT
+		     || style == GTK_HTML_PARAGRAPH_STYLE_ITEMALPHA
 		     );
 
 	gtk_widget_set_sensitive (widget, sensitive);	
@@ -177,10 +179,10 @@ font_size_changed (GtkWidget *w, GtkHTMLParagraphStyle style, GtkHTMLControlData
 {
 	if (style == GTK_HTML_FONT_STYLE_DEFAULT)
 		style = GTK_HTML_FONT_STYLE_SIZE_3;
-	cd->block_font_style_change = TRUE;
+	cd->block_font_style_change++;
 	gtk_option_menu_set_history (GTK_OPTION_MENU (cd->font_size_menu),
 				     (style & GTK_HTML_FONT_STYLE_SIZE_MASK) - GTK_HTML_FONT_STYLE_SIZE_1);
-	cd->block_font_style_change = FALSE;
+	cd->block_font_style_change--;
 }
 
 static GtkWidget *
@@ -246,9 +248,11 @@ static void
 unset_focus (GtkWidget *w, gpointer data)
 {
 	GTK_WIDGET_UNSET_FLAGS (w, GTK_CAN_FOCUS);
+	if (GTK_IS_CONTAINER (w))
+		gtk_container_forall (GTK_CONTAINER (w), unset_focus, NULL);
 }
 
-inline static void
+static void
 set_color_combo (GtkHTML *html, GtkHTMLControlData *cd)
 {
 	color_combo_set_color (COLOR_COMBO (cd->combo),
@@ -363,7 +367,7 @@ editor_toolbar_strikeout_cb (GtkWidget *widget, GtkHTMLControlData *cd)
 static void
 insertion_font_style_changed_cb (GtkHTML *widget, GtkHTMLFontStyle font_style, GtkHTMLControlData *cd)
 {
-	cd->block_font_style_change = TRUE;
+	cd->block_font_style_change++;
 
 	if (font_style & GTK_HTML_FONT_STYLE_FIXED)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cd->tt_button), TRUE);
@@ -390,7 +394,7 @@ insertion_font_style_changed_cb (GtkHTML *widget, GtkHTMLFontStyle font_style, G
 	else
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cd->strikeout_button), FALSE);
 
-	cd->block_font_style_change = FALSE;
+	cd->block_font_style_change--;
 }
 
 
@@ -493,7 +497,7 @@ editor_toolbar_unindent_cb (GtkWidget *widget,
 static GnomeUIInfo editor_toolbar_alignment_group[] = {
 	GNOMEUIINFO_ITEM_STOCK (N_("Left align"), N_("Left justifies the paragraphs"),
 				editor_toolbar_left_align_cb, GNOME_STOCK_PIXMAP_ALIGN_LEFT),
-	GNOMEUIINFO_ITEM_STOCK (N_("Center"), N_("Centers justifies the paragraphs"),
+	GNOMEUIINFO_ITEM_STOCK (N_("Center"), N_("Center justifies the paragraphs"),
 				editor_toolbar_center_cb, GNOME_STOCK_PIXMAP_ALIGN_CENTER),
 	GNOMEUIINFO_ITEM_STOCK (N_("Right align"), N_("Right justifies the paragraphs"),
 				editor_toolbar_right_align_cb, GNOME_STOCK_PIXMAP_ALIGN_RIGHT),
