@@ -160,40 +160,6 @@ html_engine_selection_pop (HTMLEngine *e)
 	html_engine_edit_selection_updater_update_now (e->selection_updater);
 }
 
-void
-html_engine_cut_and_paste_begin (HTMLEngine *e, const gchar *undo_op_name, const gchar *redo_op_name)
-{
-	html_engine_hide_cursor (e);
-	html_engine_selection_push (e);
-	html_engine_clipboard_push (e);
-	html_undo_level_begin (e->undo, undo_op_name, redo_op_name);
-	html_engine_cut (e);
-}
-
-void
-html_engine_cut_and_paste_end (HTMLEngine *e)
-{
-	if (e->clipboard) {
-		html_engine_paste (e);
-		html_engine_clipboard_clear (e);
-	}
-	html_undo_level_end (e->undo);
-	html_engine_clipboard_pop (e);
-	html_engine_selection_pop (e);
-	html_engine_show_cursor (e);
-}
-
-void
-html_engine_cut_and_paste (HTMLEngine *e, const gchar *undo_op_name, const gchar *redo_op_name,
-			   HTMLObjectForallFunc iterator, gpointer data)
-{
-	html_engine_edit_selection_updater_update_now (e->selection_updater);
-	html_engine_cut_and_paste_begin (e, undo_op_name, redo_op_name);
-	if (e->clipboard)
-		html_object_forall (e->clipboard, e, iterator, data);
-	html_engine_cut_and_paste_end (e);
-}
-
 static void
 spell_check_object (HTMLObject *o, HTMLEngine *e, gpointer data)
 {
@@ -205,6 +171,7 @@ void
 html_engine_spell_check_range (HTMLEngine *e, HTMLCursor *begin, HTMLCursor *end)
 {
 	HTMLInterval *i;
+	gboolean cited;
 
 	e->need_spell_check = FALSE;
 
@@ -215,10 +182,19 @@ html_engine_spell_check_range (HTMLEngine *e, HTMLCursor *begin, HTMLCursor *end
 	begin = html_cursor_dup (begin);
 	end   = html_cursor_dup (end);
 
-	while (html_is_in_word (html_cursor_get_prev_char (begin)))
-		if (html_cursor_backward (begin, e));
-	while (html_is_in_word (html_cursor_get_current_char (end)))
-		if (html_cursor_forward (end, e));
+	cited = FALSE;
+	while (html_selection_spell_word (html_cursor_get_prev_char (begin), &cited) || cited) {
+		if (html_cursor_backward (begin, e))
+			;
+		cited = FALSE;
+	}
+
+	cited = FALSE;
+	while (html_selection_spell_word (html_cursor_get_current_char (end), &cited) || cited) {
+		if (html_cursor_forward (end, e))
+			;
+		cited = FALSE;
+	}
 
 	i = html_interval_new_from_cursor (begin, end);
 	if (begin->object->parent != end->object->parent)
@@ -240,13 +216,28 @@ html_is_in_word (gunichar uc)
 void
 html_engine_select_word_editable (HTMLEngine *e)
 {
-	while (html_is_in_word (html_cursor_get_prev_char (e->cursor)))
+	while (html_selection_word (html_cursor_get_prev_char (e->cursor)))
 		html_cursor_backward (e->cursor, e);
 	html_engine_set_mark (e);
-	while (html_is_in_word (html_cursor_get_current_char (e->cursor)))
+	while (html_selection_word (html_cursor_get_current_char (e->cursor)))
 		html_cursor_forward (e->cursor, e);
 }
 
+void
+html_engine_select_spell_word_editable (HTMLEngine *e)
+{
+	gboolean cited, cited2;
+
+	cited = cited2 = FALSE;
+	while (html_selection_spell_word (html_cursor_get_prev_char (e->cursor), &cited))
+		html_cursor_backward (e->cursor, e);
+	html_engine_set_mark (e);
+	while (html_selection_spell_word (html_cursor_get_current_char (e->cursor), &cited2) || (!cited && cited2)) {
+		html_cursor_forward (e->cursor, e);
+		cited2 = FALSE;
+	}
+}
+
 void
 html_engine_select_line_editable (HTMLEngine *e)
 {
@@ -334,10 +325,7 @@ html_engine_clipboard_clear (HTMLEngine *e)
 HTMLObject *
 html_engine_new_text (HTMLEngine *e, const gchar *text, gint len)
 {
-	return e->insertion_url
-		? html_link_text_new_with_len (text, len, e->insertion_font_style, e->insertion_color,
-					       e->insertion_url, e->insertion_target)
-		: html_text_new_with_len      (text, len, e->insertion_font_style, e->insertion_color);
+	return html_text_new_with_len (text, len, e->insertion_font_style, e->insertion_color);
 }
 
 HTMLObject *
